/*  Lyrics plugin for XMMS by Karol Bryd <kbryd@albedo.art.pl>
 *  XMMS is Copyright (C) 1998-1999 Mikael Alm, Olle Hallnas, Thomas Nilsson and 4Front Technologies
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 */

#include <gtk/gtk.h>
#include <xmms/xmmsctrl.h>
#include <xmms/configfile.h>
#include <xmms/plugin.h>

#include "lyrics.h"
#include "list.h"

#include "support.h"

static gint my_timeout;

static struct Song *song;

static GtkWidget *creator_window = NULL;
static GtkWidget *creator_gtklist, *creator_timer_str;

static gchar *playfile;

gint secs, mins, state = FALSE;

static void open_callback(void);
static void save_callback(void);
static void *creator_set_song_pos(GtkWidget * gtklist, gint row,
				  gint column, GdkEventButton * event,
				  gpointer data);
gint creator_plugin_timeout(gpointer args);

void
creator_cleanup(void)
{
   if (creator_window)
      gtk_widget_destroy(creator_window);

   if (my_timeout)
      gtk_timeout_remove(my_timeout);
}

void
create_lyrics(void)
{
   GtkWidget *vbox, *hbox;
   GtkWidget *scrolled_window;
   GtkWidget *save_button, *open_button;

   creator_window = gtk_window_new(GTK_WINDOW_DIALOG);
   gtk_signal_connect(GTK_OBJECT(creator_window),
		      "destroy", GTK_SIGNAL_FUNC(creator_cleanup), NULL);

   gtk_window_set_default_size(GTK_WINDOW(creator_window), 200, 200);
   gtk_window_set_title(GTK_WINDOW(creator_window), _(
			"Lyrics plugin 0.0.7 by Karol Bryd <kbryd@iwaki.ahoj.pl>"));

   gtk_widget_show(creator_window);

   vbox = gtk_vbox_new(FALSE, 2);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
   gtk_container_add(GTK_CONTAINER(creator_window), vbox);
   gtk_widget_show(vbox);

   creator_timer_str = gtk_entry_new();
   gtk_box_pack_start(GTK_BOX(vbox), creator_timer_str, FALSE, TRUE, 0);
   gtk_entry_set_editable(GTK_ENTRY(creator_timer_str), FALSE);
   gtk_widget_show(creator_timer_str);

   hbox = gtk_hbox_new(FALSE, 2);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 0);
   gtk_widget_show(hbox);

   open_button = gtk_button_new_with_label(_("Open"));
   gtk_box_pack_start(GTK_BOX(hbox), open_button, FALSE, TRUE, 0);
   gtk_widget_show(open_button);
   gtk_signal_connect(GTK_OBJECT(open_button),
		      "clicked", GTK_SIGNAL_FUNC(open_callback), NULL);

   save_button = gtk_button_new_with_label(_("Save"));
   gtk_box_pack_start(GTK_BOX(hbox), save_button, FALSE, TRUE, 0);
   gtk_widget_show(save_button);
   gtk_signal_connect(GTK_OBJECT(save_button),
		      "clicked", GTK_SIGNAL_FUNC(save_callback), NULL);

   scrolled_window = gtk_scrolled_window_new(NULL, NULL);
//    gtk_widget_set_usize (scrolled_window, 250, 150);
   gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, TRUE, TRUE, 0);
   gtk_widget_show(scrolled_window);

   creator_gtklist = gtk_clist_new(1);
   gtk_container_add(GTK_CONTAINER(scrolled_window), creator_gtklist);
   gtk_widget_show(creator_gtklist);

   gtk_signal_connect(GTK_OBJECT(creator_gtklist),
		      "select_row",
		      GTK_SIGNAL_FUNC(creator_set_song_pos), NULL);

   my_timeout = gtk_timeout_add(1000, (GtkFunction)creator_plugin_timeout,
				(gpointer) creator_gtklist);
   secs = mins = 0;
}

gint
creator_plugin_timeout(gpointer args)
{
   static gchar time[32];
   long time1;

   time1 = xmms_remote_get_output_time(gp.xmms_session) / 1000;
   sprintf(time, "%02d:%02d", mins = time1 / 60, secs = time1 % 60);
   
   gtk_entry_set_text(GTK_ENTRY(creator_timer_str), time);
   return(TRUE);
}

static void
file_ok_sel(GtkWidget * w, GtkFileSelection * fs)
{
   GtkStyle *style;
   gchar *rows[1];
   gchar buf[255];
   FILE *fp;
   gint line = 0;

   fp = fopen(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)), "r");
   if(!fp)
      return;

   gtk_clist_clear(GTK_CLIST(creator_gtklist));
   while (fgets(buf, sizeof(buf), fp))
   {
      rows[0] = strdup(buf);
      rows[1] = 0;

      gtk_clist_append(GTK_CLIST(creator_gtklist), rows);
   }
   fclose(fp);
   gtk_widget_destroy((GtkWidget *) fs);
}

static void
open_callback(void)
{
   GtkObject *filew;

   filew = (GtkObject *) gtk_file_selection_new(_("File selection"));

   gtk_signal_connect(GTK_OBJECT(filew), "destroy",
		      (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filew));

   gtk_signal_connect(GTK_OBJECT
		      (GTK_FILE_SELECTION(filew)->ok_button),
		      "clicked", (GtkSignalFunc) file_ok_sel, filew);

   gtk_signal_connect_object(GTK_OBJECT
			     (GTK_FILE_SELECTION(filew)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT(filew));

   gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), "lyrics.txt");

   gtk_widget_show((GtkWidget *) filew);
}

static void
save_ok(GtkWidget * w, GtkFileSelection * fs)
{
   gchar *buf;
   FILE *out;
   gint n;

   out = fopen(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)), "w+");
   if (out)
   {
      n = 0;
      while (gtk_clist_get_text(GTK_CLIST(creator_gtklist), n++, 0, &buf))
	 fprintf(out, "%s", buf);

      fclose(out);
   }
   gtk_widget_destroy((GtkWidget *) fs);
}

static void
save_callback(void)
{
   GtkObject *filew;
   gint playlist_pos;
   gchar *playfile;
   gchar *buf;
   FILE *out;
   gint n;

   playlist_pos = xmms_remote_get_playlist_pos(gp.xmms_session);
   playfile = xmms_remote_get_playlist_file(gp.xmms_session, playlist_pos);
   buf = find_lyrics_file(playfile, 0, 0);

   filew = (GtkObject *) gtk_file_selection_new(_("File selection"));

   gtk_signal_connect(GTK_OBJECT(filew), "destroy",
		      (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filew));

   gtk_signal_connect(GTK_OBJECT
		      (GTK_FILE_SELECTION(filew)->ok_button),
		      "clicked", (GtkSignalFunc) save_ok, filew);

   gtk_signal_connect_object(GTK_OBJECT
			     (GTK_FILE_SELECTION(filew)->cancel_button),
			     "clicked", (GtkSignalFunc) gtk_widget_destroy,
			     GTK_OBJECT(filew));

   gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), buf);

   gtk_widget_show((GtkWidget *) filew);
}

static void *
creator_set_song_pos(GtkWidget * gtklist, gint row, gint column,
		     GdkEventButton * event, gpointer data)
{
   char *newline, *oldline;

   gtk_clist_get_text(GTK_CLIST(creator_gtklist), row, column, &oldline);

   newline = calloc(strlen(oldline) + 10, 1);
   sprintf(newline, "[%02d:%02d]%s", mins, secs, oldline);

   gtk_clist_set_text(GTK_CLIST(gtklist), row, column, newline);
}
