/*  Lyrics plugin for XMMS by Karol Bryd <kbryd@albedo.art.pl> XMMS is
 *  Copyright (C) 1998-1999 Mikael Alm, Olle Hallnas, Thomas Nilsson
 *  and 4Front Technologies
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.  */

#include <xmms/plugin.h>
#include <xmms/xmmsctrl.h>
#include <xmms/configfile.h>
#include <dirent.h>
#include <sys/stat.h>
#include <ctype.h>

#include "lyrics.h"
#include "list.h"
#include "conf.h"

#include "config.h"
#include "support.h"

#define TYPE_MP3_DIR 1
#define TYPE_LYRICS_DIR 2

gchar *active_color, *bg_color, *fg_color;
GdkColor active, background, foreground;
GdkColormap *colormap;
gchar last_file[255];
static int timer;
static int is_song_loaded = FALSE, song_state = TRUE;

struct Song *song;

GtkWidget *window = NULL;
GtkWidget *gtklist;
GdkFont *current_font;

gchar lyrics_dir_path[255];
gchar mp3_dir_path[255];
gchar lyrics_font[255];
gint width, height, font_height, silent = FALSE, conf = FALSE;
gint hide_mode = FALSE;
gint hide_always = FALSE;
gint look_in_mp3dir_mode = FALSE;
gint recursive_mode = FALSE;
gint desperate_mode = 0;
gint fuzzy_percent = 0;
gint delay_to_close = 40;

static void cleanup(void);
void put_to_sleep(void);

GeneralPlugin gp = {
   NULL,		/* handle */
   NULL,		/* filename */
   -1,			/* xmms_session */
   "Lyrics plugin",
   init_plugin,
   about,
   configure,
   cleanup
};

GeneralPlugin *
get_gplugin_info(void)
{
   return &gp;
}

void
init_gtk(void)
{
   GtkWidget *vbox;
   GtkWidget *scrolled_window;
   GtkWidget *button;

   gtk_set_locale();
   window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_signal_connect(GTK_OBJECT(window),
		      "destroy", GTK_SIGNAL_FUNC(put_to_sleep), NULL);

   gtk_window_set_default_size(GTK_WINDOW(window), width, height);
   gtk_window_set_title(GTK_WINDOW(window), _(
			"Lyrics plugin 0.0.7 by Karol Bryd <kbryd@iwaki.ahoj.pl>"));

   vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
   gtk_container_add(GTK_CONTAINER(window), vbox);
   gtk_widget_show(vbox);

   scrolled_window = gtk_scrolled_window_new(NULL, NULL);
   gtk_widget_set_usize(scrolled_window, 250, 25);
   gtk_container_add(GTK_CONTAINER(vbox), scrolled_window);
   gtk_widget_show(scrolled_window);

   gtklist = gtk_clist_new(1);
   gtk_container_add(GTK_CONTAINER(scrolled_window), gtklist);
   gtk_widget_show(gtklist);

   gtk_signal_connect(GTK_OBJECT(gtklist),
		      "select_row", GTK_SIGNAL_FUNC(set_song_pos), NULL);

   gtk_widget_show(window);
}

void
init_plugin(void)
{
   ConfigFile *cfg;
   GtkStyle *style;
   gchar *filename, *tmp;

   setlocale(LC_ALL, "");
   bindtextdomain(PACKAGE, LOCALEDIR);

   if (!conf)
   {
      conf = TRUE;
      filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);

      bzero(lyrics_font, sizeof(lyrics_font));
      bzero(lyrics_dir_path, sizeof(lyrics_dir_path));
      bzero(last_file, sizeof(last_file));

      cfg = xmms_cfg_open_file(filename);
      if (!cfg)
	 cfg = xmms_cfg_new();

      if (!xmms_cfg_read_int(cfg, "lyrics", "hide_mode", &hide_mode))
	 hide_mode = 0;

      if (xmms_cfg_read_string(cfg, "lyrics", "lyrics_path", &tmp))
	 strcpy(lyrics_dir_path, tmp);
      if (xmms_cfg_read_string(cfg, "lyrics", "lyrics_font", &tmp))
	 strcpy(lyrics_font, tmp);

      if (xmms_cfg_read_string(cfg, "lyrics", "width", &tmp))
	 width = atoi(tmp);
      else
	 width = DEFAULT_WIDTH;

      if (xmms_cfg_read_string(cfg, "lyrics", "height", &tmp))
	 height = atoi(tmp);
      else
	 height = DEFAULT_HEIGHT;

      if (!(xmms_cfg_read_string
	    (cfg, "lyrics", "active_color", &active_color)))
	 active_color = g_strdup("lightblue");

      if (!(xmms_cfg_read_string(cfg, "lyrics", "bg_color", &bg_color)))
	 bg_color = g_strdup("white");

      if (!(xmms_cfg_read_string(cfg, "lyrics", "fg_color", &fg_color)))
	 fg_color = g_strdup("black");

      if (!(xmms_cfg_read_int(cfg, "lyrics", "look_in_mp3dir_mode", &look_in_mp3dir_mode)))
         look_in_mp3dir_mode = 0;

      if (!(xmms_cfg_read_int(cfg, "lyrics", "recursive_mode", &recursive_mode)))
         recursive_mode = 0;

      if (!(xmms_cfg_read_int(cfg, "lyrics", "desperate_mode", &desperate_mode)))
         desperate_mode = 1;
	
      if (!(xmms_cfg_read_int(cfg, "lyrics", "fuzzy_percent", &fuzzy_percent)) ||
         fuzzy_percent > 100 ||
         fuzzy_percent < 0)
         fuzzy_percent = 20;
      xmms_cfg_write_file(cfg, filename);
      xmms_cfg_free(cfg);

      colormap = gdk_colormap_get_system();
      if (colormap)
      {
	 gdk_color_parse(active_color, &active);
	 gdk_color_alloc(colormap, &active);
	 gdk_color_parse(bg_color, &background);
	 gdk_color_alloc(colormap, &background);
	 gdk_color_parse(fg_color, &foreground);
	 gdk_color_alloc(colormap, &foreground);
      }
   }

   init_gtk();

   timer = gtk_timeout_add(200, plugin_timeout, (gpointer) gtklist);

   current_font = gdk_fontset_load(lyrics_font);
   if (current_font)
      get_font_height();
   else
   {
      strcpy(lyrics_font, "fixed");
      current_font = gdk_fontset_load("fixed");
      if (current_font)
	 get_font_height();
   }
   silent = FALSE;
}

static void
cleanup(void)
{
   silent = TRUE;

   if (timer)
   {
      gtk_timeout_remove(timer);
      timer = 0;
   }

   if (window)
   {
      gtk_widget_destroy(window);
      window = NULL;
   }

   if (conf_win)
   {
      gtk_widget_destroy(conf_win);
      conf_win = NULL;
   }
}

void
put_to_sleep(void)
{
   silent = TRUE;
   window = NULL;
}

void
get_font_height(void)
{
   if (!gtklist)
      return;

   font_height = current_font->ascent + abs(current_font->descent);

   gtk_clist_set_row_height(GTK_CLIST(gtklist), font_height + 2);
}

void
set_line(gint time)
{
   struct LyricsLine *line;
   GdkWindow *gdkwindow;
   GtkStyle *style;
   static gint lastrow = 0;
   gint found;

   if (!gtklist || !song || silent)
      return;

   /* remember the window's size */
   gdkwindow = gtk_widget_get_parent_window(gtklist);
   if (gdkwindow)
      gdk_window_get_size(gdkwindow, &width, &height);

   /* find the current line */
   line = song->head;
   found = FALSE;
   while (line)
   {
      if (line->line_time == time)
      {
	 found = TRUE;
	 break;
      }
      line = line->next;
   }
   /* if not found then simply exit */
   if (!found)
      return;

   /* change color of current line from active to non active */
   style = gtk_style_new();
   if (style)
   {
      gdk_font_unref(style->font);
      style->font = current_font;
      style->base[GTK_STATE_NORMAL] = background;
      style->bg[GTK_STATE_NORMAL] = active;
      style->fg[GTK_STATE_NORMAL] = foreground;
      gtk_clist_set_row_style(GTK_CLIST(gtklist), lastrow, style);
   }

   /* move to the new line */
   gtk_clist_moveto(GTK_CLIST(gtklist), line->line_number, 0, 0.5, 0.0);

   /* and set color to active */
   style = gtk_style_new();
   if (style)
   {
      gdk_font_unref(style->font);
      style->font = current_font;
      style->base[GTK_STATE_NORMAL] = active;
      style->bg[GTK_STATE_NORMAL] = active;
      style->fg[GTK_STATE_NORMAL] = foreground;
      gtk_clist_set_row_style(GTK_CLIST(gtklist), line->line_number, style);
   }

   lastrow = line->line_number;
}


void
redraw_list(void)
{
   struct LyricsLine *line;

   if (!gtklist || !song || silent)
      return;

   get_font_height();
   gtk_clist_clear(GTK_CLIST(gtklist));

   /* walk through the lyric list and append nodes to the CList */
   line = song->head;
   for (; line; line = line->next)
   {
      GtkStyle *style;

      if (!line->rep)
	 gtk_clist_append(GTK_CLIST(gtklist), &line->buffer);

      style = gtk_style_new();
      if (style)
      {
	 gdk_font_unref(style->font);
	 style->font = current_font;
	 style->base[GTK_STATE_NORMAL] = background;
	 style->bg[GTK_STATE_NORMAL] = active;
	 style->fg[GTK_STATE_NORMAL] = foreground;

	 gtk_clist_set_row_style(GTK_CLIST(gtklist),
				 line->line_number, style);
      }
   }
   set_line(xmms_remote_get_output_time(gp.xmms_session) / 1000);
}

/*
 * Processing stages (by Michal Kochanowicz <mkochano@pld.org.pl>):
 * 1. Conversion to lower-case.
 * 2. All digits, commas, underscores, brackets and so converted to
 *    spaces. Ampersand (&) is left untouched (see STAGE 5). So are minuses
 *    (they are used to separate song title from author's name).
 * 3. All remaining, non [a-z&- ] characters converted to dots. They can be
 *    diacratical characters, which are somtimes replaced by similar [a-z]
 *    characters, so we can't be sure what to do with them. Dots, as in
 *    regular expression match any character. See fuzzystrcmp() below. This
 *    may look like it replicates functionality of fuzzines type 3, but we
 *    don't want fuzzines_percent to have any influence on matchin language
 *    specific chars.
 * 4. "the " strings replaced with "    ". This word is quite oftes missused ;)
 * 5. "and " strings replaced with "&    ".
 * 6. All (yes, ALL spaces) removed. Consider R.E.M. Or maybe R.e.m. Or REM.
 *    I've seen all of them, and I'm sure there are people using all other
 *    possible variants :)
 *
 * Sample of processing (note polish 's with an accent' as the fifth character
 * of mp3 name being often replaced with plain 's' as in case of lyrics file
 * name - BTW: Kubu Puchatek == Winie the Pooh ;) ):
 *   Kubu Puchatek & Friends - The Theme.mp3	02.Kubus_Puchatek_and_friends-theme.txt
 * 1 kubu puchatek & friends - the theme	02.kubus_puchatek_and_friends-theme
 * 2 kubu puchatek & friends - the theme	   kubus puchatek and friends-theme
 * 3 kubu. puchatek & friends - the theme	   kubus puchatek and friends-theme
 * 4 kubu. puchatek & friends -     theme	   kubus puchatek and friends-theme
 * 5 kubu. puchatek & friends -     theme	   kubus puchatek &   friends-theme
 * 6 kubu.puchatek&friends-theme		kubuspuchatek&friends-theme
 *
 * And what we got now?
 * 		kubu.puchatek&friends-theme
 * 		kubuspuchatek&friends-theme
 * Match! Booth strings have been converted so they match, and (hopefully)
 * still contain enough data to not be missmatched with soumething else :)
 */
char *process_filename(char *s)
{
   char *buf, *orig, *c, *c2;
   
   orig = buf = calloc(1, strlen(s) + 1);

   /* STAGES 1, 2 & 3 */
   while(*s)
   {
      /* STAGE 1 */
      *buf = tolower(*s);
      /* STAGE 2 */
      if(strchr("`~!@#$%^*()_-+={}[]:;\"'<>?,./0123456789", *buf))
         *buf = ' ';
      /* STAGE 3 */
      if((*buf < 'a' || *buf > 'z') && *buf != '&' && *buf != '-' && *buf != ' ')
         *buf = '.';
      /* Next char */
      s++;
      buf++;
   }

   /* STAGE 4 */
   while((c = strstr(orig, "the ")))
      memset(c,  ' ', 3);
   /* STAGE 5 */
   while((c = strstr(orig, "and ")))
   {
      *c = '&';
      memset(c+1,' ', 2);
   }

   /* STAGE 6 */
   /* 
    * Removing spaces:
    * c points after last byte copied, c2 to byte which may be copied. c2 is
    * incrased in every iteration. c is incrased and copying occurs only
    * if source byte is not a space.
    */
   c = orig; c2 = orig;
   while(*c2)
   {
      if(*c2 != ' ')
         *(c++) = *c2;
      c2++;
   }
   *c = '\0';
   *buf = 0;

   return(orig);
}

/*
 * Fuzzy comparsion (by Michal Kochanowicz <mkochano@pld.org.pl>)
 * 
 * fuzzycompare():
 *    Compares two strings (song/lyrics files). At first compares them
 *    by calling dofuzzystrcmp(). If it fails, and "desperate" mode is
 *    selected, extracts titles from file names and fuzzy-compares them.
 *
 * dofuzzystrcmp()
 *    Function compares two strings. Dot match any character. Additional
 *    parameter gives acceptable level of difference.
 *
 *    Function compares given strings byte by byte. If it finds difference
 *    (note that dot are treated as matches) and fuzzines_left is not zero
 *    it tries three different fuzzineses:
 *    1. additional character in first string. So it is skipped. Function
 *       call itself giving as arguments next char in first string, current in
 *       second and fuzzines_left decrased by one.
 *    2. additional character in second string. So it is skipped. Function
 *       call itself giving as arguments current char in first string, next in
 *       second and fuzzines_left decrased by one.
 *    3. spelling mistake in one or booth strings. One char is skipped in booth
 *       strings. Function calls itself with next characters of booth strings
 *       and fuzzines_left as arguments.
 *    If one of tests return '0' it means that correct fuzzines type has been
 *    tried, and routine (possible using up to fuzzines_level of recurention)
 *    was able to match booth strings. In such case '0' is returned. If other
 *    case next fuzzines type is tried. If all tries fail, result of last is
 *    returned.
 * 
 * extract_title()
 *    At first, part of the string starting at last dot is thrown away. Then,
 *    if string contains "minus", the part of string from beginning up to
 *    "minus" is thrown away. Function allocates new string on heap and it is
 *    up to user to release memory.
 */
#ifdef DEBUG
int call_count = 0;
#endif
int dofuzzystrcmp(const char *s1, const char *s2, int fuzzines_left)
{
   int i;

#ifdef DEBUG
   call_count++;
#endif

//   dprintf(" %04d  fuzzystrcmp(%s\n"
//	   "                   %s)\n", fuzzines_left, s1, s2);
   while(*s1 && *s2)
   {
      if(*s1 != *s2 && *s1 != '.' && *s2 != '.')
      {
         if(!fuzzines_left)
            break;
	 if(*(s1+1))		/* type 1 */
            if(!dofuzzystrcmp(s1+1, s2, fuzzines_left-1))
               return 0;
	 if(*(s2+1))		/* type 2 */
            if(!dofuzzystrcmp(s1, s2+2, fuzzines_left-1))
               return 0;
	 if(*(s1+1) && *(s2+1))	/* type 3 */ {
            return dofuzzystrcmp(s1+1, s2+2, fuzzines_left-1);
	 }
      }
      s1++;
      s2++;
   }
   //dprintf(": %d\n", (int)(*s1 - *s2));
   if((i = (strlen(s1) - strlen(s2))))
      if(abs(i) <= fuzzines_left)
         return 0;
   return *s1 - *s2;
}

/* See comment before dofuzzystrcmp() above. */
char *extract_title(const char *s)
{
   char *c = index(s, '-'), *ss;

   if(!(c++))
      (const char *)c = s; /* We will not modify *s... */

   ss = strdup(c);
   if((c = rindex(ss, '.')))
      *c = '\0';

   dprintf("EXTRACT %s -> %s\n", s, ss);
   return ss;
}

/* See comment before dofuzzystrcmp() above. */
int fuzzystrcmp(const char *s1, const char *ps1, const char *s2, const char *ps2, int desperate, int fuzzy_percent)
{
   int res, fuzzines_left;
   char *ss1, *pss1, *ss2, *pss2;

   dprintf("%s, %s, %s, %s, %d, %d\n", s1, ps1, s2, ps2, desperate, fuzzy_percent);
   if(desperate) {
      ss1 = extract_title(s1);
      ss2 = extract_title(s2);
      pss1 = process_filename(ss1);
      pss2 = process_filename(ss2);
      /* Calculate fuzzines_left for titles. */
      fuzzines_left = (fuzzy_percent * ((strlen(pss1) + strlen(pss2)) >> 1)) / 100;
      res = dofuzzystrcmp(pss1, pss2, fuzzines_left);

      free(pss2);
      free(pss1);
      free(ss2);
      free(ss1);
   } else {
      /* Count fuzzy_percent percents of average length of given strings. */
      fuzzines_left = (fuzzy_percent * ((strlen(ps1) + strlen(ps2)) >> 1)) / 100;

      res = dofuzzystrcmp(ps1, ps2, fuzzines_left);
   }
   return res;
}

char *remove_ext(char *file)
{
   static char file_copy[255];
   char *pos;

   strncpy(file_copy, file, sizeof(file_copy));
   file_copy[sizeof(file_copy)-1] = '\0';

   if((pos = strrchr(file_copy, '.')))
      *pos = '\0';

   return file_copy;
}

char *join_dir_and_file(const char *dir, const char *file)
{
   static char buf[512];
   strncpy(buf, dir, sizeof(buf));
   buf[sizeof(buf)-1] = '\0';
   if(buf[strlen(buf)-1] != '/')
      strncat(buf, "/", sizeof(buf) - strlen(buf));
   buf[sizeof(buf)-1] = '\0';
   
   strncat(buf, file, sizeof(buf) - strlen(buf));
   buf[sizeof(buf)-1] = '\0';
   return buf;
}

char *find_file_in_dir(char *path, char *file, int desperate)
{
   char *ext[] = {".txt", ".lrc", 0};
   DIR *dir;

   dprintf("find_file_in_dir(%s, %s)\n", path, file);
   dir = opendir(path);
   if(dir)
   {
      struct dirent *dirent;
      char *file_without_ext = strdup(remove_ext(file));
      char *processed_filename = process_filename(file_without_ext);

      while((dirent = readdir(dir)))
      {
	 char *processed_direntry, *e;
	 int ext_num = 0;
	 struct stat info;
	 char *p;

	 if(dirent->d_name[0] == '.')
	    continue;
	 
	 p = join_dir_and_file(path, dirent->d_name);
	 if(stat(p, &info))
            dprintf("STAT(%s) ERROR!\n", p);

	 /* If it is a direcotory, dive int it. */
	 if(S_ISDIR(info.st_mode)) {
            char *res;
	    char *dir = strdup(p);
	   
	    if(recursive_mode) {
               res = find_file_in_dir(dir, file, desperate);
	       free(dir);
	    
               if(res)
                  return res;
	    }

	    continue;
	 }

	 /* Do not even consider loading too large (>100kB) files. */
	 if(info.st_size > 100*1024)
            continue;

	 /* Does it have any extention? */
	 if(!(e = strrchr(dirent->d_name, '.')))
            continue;

	 while(ext[ext_num]) {
            if(!strcasecmp(e, ext[ext_num])) {
	       processed_direntry = process_filename(remove_ext(dirent->d_name));

               if(fuzzystrcmp(
                  dirent->d_name, processed_direntry,
                  file_without_ext, processed_filename,
                  desperate, fuzzy_percent) == 0)
	       {
                  char *res = strdup(join_dir_and_file(path, dirent->d_name));
		  
                  closedir(dir);
                  free(processed_direntry);
                  return(res);
               }
               free(processed_direntry);
               break;
	    }
	    ext_num++;
         }
      }
      closedir(dir);
      free(processed_filename);
      free(file_without_ext);
   }
   return(NULL);
}

char *find_lyrics_file(char *title, int type, int desperate)
{
   char *name;

#ifdef DEBUG
   call_count = 0;
#endif
   dprintf("FIND_LYRICS_FILE(%s, %d)\n", title, type);
   if(type == TYPE_MP3_DIR)
   {
      name = find_file_in_dir(mp3_dir_path, title, desperate);
      if(name)
      {
#ifdef DEBUG
         dprintf("FIND_LYRICS_FILE(%s, %d) - %d calls to dofuzzystrcmp(): %s\n", title, type, call_count, name);
#endif
	 return(name);
      }
   }
   else
   {
      name = find_file_in_dir(lyrics_dir_path, title, desperate);
      if(name)
      {
#ifdef DEBUG
         dprintf("FIND_LYRICS_FILE(%s, %d) - %d calls to dofuzzystrcmp(): %s\n", title, type, call_count, name);
#endif
	 return(name);
      }
   }
#ifdef DEBUG
   dprintf("FIND_LYRICS_FILE(%s, %d) - %d calls to dofuzzystrcmp()\n", title, type, call_count);
#endif
   return(NULL);
}

void
do_load_file(char *playfile_full)
{
   char *lyricsfile;
   char *filename_begin;
   char *playfile;

   strncpy(last_file, playfile_full, sizeof(last_file));

   filename_begin = rindex(playfile_full, '/');
   if(filename_begin)
   {
      int a = strlen(playfile_full);

      while((playfile_full[a] != '/') && (a >= 0))
	 a--;

      /* This call WILL NOT TERMINATE string with null! */
      strncpy(mp3_dir_path, playfile_full, a);
      mp3_dir_path[a] = '\0';

      playfile = filename_begin + 1;
   }
   else
   {
      strcpy(mp3_dir_path, "./");
      playfile = playfile_full;
   }
   
   /* try to get filename. */
   lyricsfile = find_lyrics_file(playfile, TYPE_LYRICS_DIR, 0);
   if(!lyricsfile && look_in_mp3dir_mode)
      lyricsfile = find_lyrics_file(playfile, TYPE_MP3_DIR, 0);

   /* If stil nothing, and user requested despeare mode, go for it. */
   if(!lyricsfile && desperate_mode) {
      dprintf("DESPERATE MODE >>>>>>>>>>>>>>\n");
      lyricsfile = find_lyrics_file(playfile, TYPE_LYRICS_DIR, 1);
      if(!lyricsfile && look_in_mp3dir_mode)
         lyricsfile = find_lyrics_file(playfile, TYPE_MP3_DIR, 1);
      dprintf("DESPERATE MODE <<<<<<<<<<<<<<\n");
   }

   is_song_loaded = FALSE;

   if(lyricsfile)
   {
      /* try to load lyrics */
      if ((song = read_lyrics_file(lyricsfile)))
      {
	 redraw_list();
	 is_song_loaded = TRUE;
      }
      free(lyricsfile);
   }
   else
   {
      static gchar tmp[255];
      static gchar *error_str[] = { "x", };
      
      sprintf(tmp, _("Lyrics for song '%s' not found"), playfile_full);
      error_str[0] = tmp;
      gtk_clist_append(GTK_CLIST(gtklist), error_str);
   }
}

void
load_song(void)
{
   struct LyricsLine *line;
   gint playlist_pos;
   gchar *playfile;

   /* is XMMS playing? */
   if (xmms_remote_is_playing(gp.xmms_session))
   {
      /* if yes, then get current song's position in the playlist */
      /* and it's filename */
      playlist_pos = xmms_remote_get_playlist_pos(gp.xmms_session);
      playfile = xmms_remote_get_playlist_file(gp.xmms_session, playlist_pos);
      /* if current filename and position differs from the older ones then reload lyrics */
      if (strcmp(last_file, playfile))
      {
	 if (!window)
	 {
	    init_gtk();
	    silent = FALSE;
	 }

	 gtk_clist_clear(GTK_CLIST(gtklist));

	 // first try to find lyrics file in the given path
	 do_load_file(playfile);
      }
   }
}

gint
plugin_timeout(gpointer args)
{
   /* check if there wasn't a song change */
   load_song();
   if (hide_mode)
   {
      if (song_state && xmms_remote_is_playing(gp.xmms_session) && (window == NULL))
      {
	 init_gtk();
	 silent = FALSE;
	 last_file[0] = 0;
      }
      else if (!xmms_remote_is_playing(gp.xmms_session) && window)
      {
	 last_file[0] = 0;
	 gtk_widget_destroy(window);
	 window = NULL;
	 put_to_sleep();
	 song_state = FALSE;
      }
	
      // When song is not found then wait for some seconds and then
      // close the window
      if (!is_song_loaded)
      {
	 if (delay_to_close)
	    delay_to_close--;
	 else
	 {
	    if (window)
	    {
	       gtk_widget_destroy(window);
	       window = NULL;
	       put_to_sleep();
	       delay_to_close = 15;
	       song_state = FALSE;
	    }
	 }
      }
   }
   
   /* and update display */
   set_line(xmms_remote_get_output_time(gp.xmms_session) / 1000);
   return (TRUE);
}

void *
set_song_pos(GtkWidget * gtklist, gint row, gint column,
	     GdkEventButton * event, gpointer data)
{
   struct LyricsLine *line;

   /* let's find out what line was selected */
  
   if (!is_song_loaded)
      return;

   line = song->head;
  
   while (row != line->line_number)
   {
      line = line->next;
      if (!line)
	 break;
   }

   /* now jump there */
   xmms_remote_jump_to_time(gp.xmms_session, line->line_time * 1000);
}

#if 0
int
main(void)
{
   /*char *c1 = "Kubu Puchatek & Friends - The Theme";
   char *c2 = "02.Kubus_Puchatek_and_friends-theme";*/
   char *c1 = "you are so beautiful.TXT";
   char *c2 = "Joe_Cocker-You_are_so_beatifull.mp3";

   char *pc1 = process_filename(c1);
   char *pc2 = process_filename(c2);

   printf("\"%s\"\n\"%s\"\n\"%s\"\n\"%s\"\n%d\n", c1, pc1, c2, pc2, fuzzystrcmp(c1, pc1, c2, pc2, 1, 20));
   return 0;
}
#endif
